program read_netCDF_f

        use netcdf

        implicit none

        TYPE TYPE_NC_DIM
            INTEGER :: dim_len
            CHARACTER(256) :: dim_name
        END TYPE TYPE_NC_DIM

        TYPE TYPE_NC_VAR
            CHARACTER(256) :: var_name
            INTEGER :: var_num_dims, var_type, var_dims(1024)
        END TYPE TYPE_NC_VAR

        INTEGER :: ncid, latid, lonid, refid, restatus, nprof_idx, nprofid ,nbinid ,len
        INTEGER :: ndims_p, nvars_p, dimids(1024), include_parents, varids(1024)
        INTEGER :: idx, nbin_len, nprof_len, i
        INTEGER :: vartype, varndims, vardimids(1024), varnatts
        !REAL*4, DIMENSION(37081) :: data_lat 
        !REAL*4, DIMENSION(37081) :: data_lon
        !REAL*4, DIMENSION(37081,83) :: data_ref
        CHARACTER(256) :: dimname
        CHARACTER(256) :: varname
        CHARACTER(256) :: work_buf

        TYPE(TYPE_NC_DIM), allocatable :: nc_dim(:)
        TYPE(TYPE_NC_VAR), allocatable :: nc_var(:)
        REAL*4,allocatable :: data_lat(:)
        REAL*4,allocatable :: data_lon(:)
        REAL*4,allocatable :: data_ref(:,:)
        CHARACTER(10) :: valiable_types(0:12) = (/"nat","byte","char","short","int","float",&
            "double","ubyte","ushort","uint","int64","uint64","string"/)

        ! Open existing netCDF dataset     
        restatus = nf90_open("CS20062762001_2B-GEOPROF_VERTICAL-MEAN_V1.0.nc", nf90_NoWrite, ncid)

        ! NetCDF inquiry ***
        restatus = nf90_inq_dimids( ncid, ndims_p, dimids, include_parents)
        restatus = nf90_inq_varids( ncid, nvars_p, varids)

        ! allocate dimension datas ***
        allocate(nc_dim(ndims_p))
        allocate(nc_var(nvars_p))

        ! Dimension name and size acquisition ***
        do idx = 1, ndims_p
            restatus = nf90_inquire_dimension( ncid, idx, dimname, len)

            nc_dim(idx)%dim_len=len
            nc_dim(idx)%dim_name=dimname

            if (nc_dim(idx)%dim_name .eq. "nbin") then
               nbin_len = nc_dim(idx)%dim_len 
            elseif (nc_dim(idx)%dim_name .eq. "nprof") then
               nprof_len = nc_dim(idx)%dim_len 
            endif

        enddo

        ! get all var defines ***
        do idx = 1, nvars_p
            restatus = nf90_inquire_variable( ncid, idx, varname,&
                        vartype, varndims, vardimids, varnatts)
            
            nc_var(idx)%var_name=varname
            nc_var(idx)%var_type=vartype
            nc_var(idx)%var_num_dims=varndims
            do i = 1, varndims
                nc_var(idx)%var_dims(i) = vardimids(i)
            enddo

            if (nc_var(idx)%var_name .eq. "lat") then
                allocate(data_lat(nprof_len))
                latid = idx
            elseif (nc_var(idx)%var_name .eq. "lon") then
                allocate(data_lon(nprof_len))
                lonid = idx
            elseif (nc_var(idx)%var_name .eq. "ref") then
                allocate(data_ref(nprof_len,nbin_len))
                refid = idx
            endif
        enddo

        ! Get variable IDs
        ! restatus = nf90_inq_varid( ncid, "lat", latid)
        ! restatus = nf90_inq_varid( ncid, "lon", lonid)
        ! restatus = nf90_inq_varid( ncid, "ref", refid)

        ! Print dimensions
        write(*,*) "dimensions:"
        do idx = 1, ndims_p
            write(*,100), idx, trim(nc_dim(idx)%dim_name), nc_dim(idx)%dim_len
        enddo
   100  format('   dim_id : ',(i3),'   ',(a6),' : ',(i))

        ! Print variables
        write(*,*) "variables:"
        do idx = 1, nvars_p
        work_buf = ""
            do i = 1, nc_var(idx)%var_num_dims
                if (i .ne. 1) then
                    work_buf = trim(work_buf) // ","
                endif
                work_buf = trim(work_buf) // trim(nc_dim(nc_var(idx)%var_dims(i))%dim_name)
            enddo
            write(*,200), idx,&
                valiable_types(nc_var(idx)%var_type),&
                    trim(nc_var(idx)%var_name), trim(work_buf)
        enddo
   200  format('   var_id : ',(i3),'   ',(a8),'  ',(a10),'(',(a),')')

        ! Get values of variables
        restatus = nf90_get_var(ncid, latid, data_lat)
        restatus = nf90_get_var(ncid, lonid, data_lon)
        restatus = nf90_get_var(ncid, refid, data_ref)

        ! Print get values 
        write(*,*)  "data:"
        do nprof_idx = 1, 5 
            write(*,300)  "lat", nprof_idx, data_lat(nprof_idx)
            write(*,300)  "lon", nprof_idx, data_lon(nprof_idx)
            do i =1, 3
                write(*,400)  "ref", nprof_idx, i, data_ref(nprof_idx,i)
            enddo
        enddo
   300  format('   ',(a3),'[',(i3),']=',(f20.6))
   400  format('   ',(a3),'[',(i3),'][',(i3),']=',(f15.6))

        ! close netcdf dataset 
        restatus = nf90_close(ncid)

        deallocate(nc_dim)
        deallocate(nc_var)
end program read_netCDF_f

